/* Moorhuhn-Lasertag V0.3, 14.06.2017
 * von Sebastian Voss
 */

#include <Moorhuhn.h> // Bibliothek für die Moorhühner
#include <Vector2.h>
// Bibliotheken für das LCD-Display
#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x27,20,4); // LCD mit SDA an A4, SCL an A5; 20 Spalten, 4 Zeilen

// Definition der LED-Kette
#define led_kette 0

// Definition der Pins für die LEDs, die das Ziel angeben
#define led_0 1
#define led_1 2
#define led_2 3
#define led_3 4
#define led_4 5
#define led_5 6

// Definition des Piezo-Summers für das akustische Feedback
#define buzzer 7

// Definition der Servos für die Bewegung der Huehner
#define servonr_0 8
#define servonr_1 9
#define servonr_2 10
#define servonr_3 11
#define servonr_4 12
#define servonr_5 13

// Referenzwerte für die LDR
// wenn das Spiel gestartet wird, wird ein Countdown heruntergezählt und am Ende des Countdowns, also direkt vor dem Spiel werden die Referenzwerte gemessen
// damit die aktuelle Umgebungshelligkeit als Referenz dient und damit niemand mehr direkt vor dem Spiel steht und den Referenzwert durch seinen Schatten
// verändert
int ldr_ref;

// Array für die Punkte der Spieler (maximal 2)
int a;
int b;
int counter[]={a,b};
int sp=1; // Spieleranzahl
float react_time=4.0; // Reaktionszeit zum Abschießen
int hbew=5; // Häufigkeit der Bewegungen

// Erstellen der Moorhühner
Moorhuhn huhn0(0,led_0,servonr_0,buzzer,false); // das Huhn bekommt eine Nummer (für den LDR), einen Pin für eine LED, einen Pin für einen Servo und einen Pin für den Buzzer  zugeordnet
Moorhuhn huhn1(1, led_1, servonr_1,buzzer,false);
Moorhuhn huhn2(2, led_2, servonr_2, buzzer, false);
Moorhuhn huhn3(3, led_3, servonr_3, buzzer, false);
Moorhuhn huhn4(4, led_4, servonr_4, buzzer, false);
Moorhuhn huhn5(5, led_5, servonr_5, buzzer, false);

Vector<Moorhuhn> huehner;

void setup() {
  // LCD initialisieren
  lcd.init();  //initialize the lcd
  lcd.backlight();  //open the backlight
  lcd.setCursor(0,0);
  // Hintergrundbeleuchtung (LED-Kette)
  pinMode(led_kette, OUTPUT);
  digitalWrite(led_kette, HIGH);
  // Alle Setup-Einstellungen für die Hühner werden im Constructor vorgenommen
  huehner.push_back(huhn0);
  huehner.push_back(huhn1);
  huehner.push_back(huhn2);
  huehner.push_back(huhn3);
  huehner.push_back(huhn4);
  huehner.push_back(huhn5);
  //Serial.begin(9600);
  /*Serial.print("Vector: ");
  Serial.print(huehner[0].nr); // funktioniert
  Serial.println(huehner[1].nr);*/
}

// ********************************************************** Spiele *********************************************************************************************

/* Am Anfang werden alle Moorhühner auf lebend gestellt und dann müssen diese möglichst schnell abgeschossen werden. Zurückgegeben wird die benötigte Zeit in Sekunden, gerundet
 * auf zwei Nachkommastellen.
 * Funktionsweise: 
 */
void killemall () {
  // bisherige Überlegungen/Skizzen
  // Hühner zum Leben erwecken: alive auf true und LED an
  //for(int i=0; i<=1; i++) {
  //  huehner[i].live(true);
  //}
  huehner[0].live(true);  // funktioniert nicht bzw. bleibt auf die cpp-Datei beschränkt - innerhalb der aufgerufenen Funktion wird die Variable umgestellt
  huehner[1].live(true);  // dagegen funktioniert das Anstellen der LED als Zeichen für das Leben
  // genauso wenig funktioniert huehner[0].alive=true;
  //Serial.println("Hauptdatei:");
  //Serial.println(huhn0.alive);
  //Serial.println(huhn1.alive);
  huhn0.live(true);  // mit dieser Variante funktioniert sowohl das globale Umstellen der Variable alive als auch das Anstellen der LED
  huhn1.live(true);
  //Serial.println("Klassische Änderung");
  //Serial.println(huehner[0].alive); // funktioniert richtig
  //Serial.println(huhn1.alive);
  ldr_ref=analogRead(A0);
  int timer=0;
  while (!huehner[0].shot(ldr_ref) && !huehner[1].shot(ldr_ref) // funktioniert
    //for (int i=0; i<huehner.size(); i++) {!huehner[i].shot(ldr_ref);} // solange kein Huhn abgeschossen wird - wie mit && verbinden?!
    ) {
    if (timer%2 == 0 && huhn0.alive) huehner[0].fly(); // funktioniert
    if (timer%2 == 1 && huhn1.alive) huehner[1].fly();
    timer++;
    delay(100);
  }
  for(int i=0; i<=1; i++) {
    if (huehner[i].shot(ldr_ref)) {
      huehner[i].live(false); // Ausstellen der LED funktioniert
      //Serial.println("Sterben:");
      //Serial.println(huhn0.alive); // Umstellen der alive-Variable funktioniert nicht
      //Serial.println(huhn1.alive);
      huehner[i].die(); // Ton für Sterben funktioniert
    }
  }
  // Danach wieder zurück zu Schleife mit den übrig gebliebenen Hühnern...
  
  do {} while(1);
}

/* Klassisches Moorhuhn: Es taucht ein Moorhuhn an zufälliger Stelle auf und muss möglichst schnell abgeschossen werden.
 *  Die Zeit vom Auftauchen bis zum Abschuss wird gezählt. Je nach Schnelligkeit gibt es 50, 30 oder 10 Punkte oder die Runde ist beendet. 
 *  Die Grenzen für die Punkte werden je nach eingestellter Reaktionszeit gesetzt (Schwierigkeit 1). Wenn die Runde nicht beendet, sondern Punkte vergeben wurden,
 *  geht es weiter mit dem nächsten Moorhuhn. Zurückgegeben wird die Anzahl der Punkte. Außerdem kann man das Spiel zu zweit (abwechselnd) spielen.
 *  Die Bewegungshäufigkeit hbew gibt an, wie oft sich das Huhn bewegen soll (Schwierigkeit 2). Wenn hbew=1 ist, dann bewegt sich das Huhn alle 0.1 Sekunden,
 *  bei hbew=2 alle 0.2 Sekunden usw. Empfehlenswert ist also hbew von 10 bis 20.
 */

void classic (int sp, float react_time, int hbew) {
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Spielstart in ..."); // Countdown
  for (int i=3; i>0; i--) {
    lcd.setCursor(9,1);
    lcd.print(i);
    delay(1000);
  }
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Moorhuhn klassisch"); // Überschrift
  //for(int i=0; i<2; i++) { // Punktestände auf null
  //  counter[i]=0;
  //}
  a=0; // Punktestände auf null
  b=0;
  int znr=10; // Zufällige Nummer eines Huhns - Start mit 10, damit der Zufallsgenerator unten diese Zahl auf jeden Fall überschreibt
  float zeit; // gemessene Reaktionszeit
  int schleife=1; // Schleifenzähler für abwechselndes Spielen
  do {
    int dran = schleife % sp; // ermittle, welcher Spieler dran ist
    lcd.setCursor(0,1);
    lcd.print("Spieler " + String(dran+1) + " ist dran.");
    int z;
    do {
      z=random(6); //generiere neue Zufallszahl von 0 bis 5
    } while (znr==z); // solange die neue Zufallszahl gleich der alten Zufallszahl ist
    znr=z; // Wenn die (neue) Zufallszahl nicht gleich der alten Zufallszahl ist, dann übernehme sie
           // Dadurch wird vermieden, dass dasselbe Huhn zwei Mal hintereinander abgeschossen werden soll
    ldr_ref=analogRead(A0); // Referenzwert für die Umgebungshelligkeit einlesen
    huehner[znr].live(true); // Huhn zum Leben erwecken: LED an
    unsigned long start = millis(); // Start der Zeitmessung
    int timer;
    // Solange das Huhn nicht abgeschossen wurde, soll es sich immer wieder bewegen
    // hbew = 1 -> Huhn bewegt sich jede zweite Runde, also alle 100 Millisekunden
    // hbew = 2 -> Huhn bewegt sich jede vierte Runde, also alle 200 Millisekunden
    // hbew = n -> Huhn bewegt sich jede 2n-te Runde, also alle n*100 Millisekunden
    while(!huehner[znr].shot(ldr_ref)) {
      if (timer%(2*hbew) ==0) huehner[znr].fly();
      delay(50);
      timer++;
    }
    unsigned long ende = millis(); // Ende der Zeitmessung
    huehner[znr].live(false); // LED aus
    huehner[znr].die(); // Sterbeton
    zeit = (ende - start)/1000.; // Reaktionszeit berechnen
    // Punkte ermitteln
    if (zeit <= 0.33*react_time) {
      //counter[dran]+=50;
      if (dran == 0) {
        a+=50;
      } else {
        b+=50;
      }
      lcd.setCursor(8,2);
      lcd.print("+50!");
    } else if (0.33*react_time < zeit && zeit <= 0.66*react_time) {
      //counter[dran]+=30;
      if (dran == 0) {
        a+=30;
      } else {
        b+=30;
      }
      lcd.setCursor(8,2);
      lcd.print("+30!");
    } else if (0.66*react_time < zeit && zeit <=react_time) {
      //counter[dran]+=10;
      if (dran == 0) {
        a+=10;
      } else {
        b+=10;
      }
      lcd.setCursor(8,2);
      lcd.print("+10!");
    } else if (react_time < zeit) {
      lcd.setCursor(5,2);
      lcd.print("Zu langsam!");
    }
    // Punkte ausgeben
    lcd.setCursor(0,3);
    lcd.print("Punkte v. Sp." + String(dran+1) + ":");
    //lcd.print(counter[dran]);
    if (dran ==0) {
      lcd.print(a);
    } else {
      lcd.print(b);
    }
    delay(2000);
    lcd.setCursor(0,2);
    lcd.print("                    ");
    schleife++;
  } while (zeit<=react_time);
}

// **************************************************************** Menüführung *******************************************************************************
int lcd_key     = 0;
#define t1   101
#define t2   102
#define t3   103
#define no_t 100

int read_buttons() {
 int key_in = analogRead(A3);  // Wert für die Taster einlesen
 if (key_in > 440) return no_t; // Wenn alle offen sind, ist der Widerstand hoch
 if (key_in > 410) return t1;  // t1 liegt bei 426
 if (key_in > 370) return t2; // t2 liegt bei 393
 if (key_in > 310) return t3;  // t3 liegt bei 342
 return no_t;  // wenn alle anderen nicht hinhauen, ist kein Taster gedrückt (eigentlich sind aber mehrere gedrückt)
}

/* Einstellung der Reaktionszeit, die man maximal hat, um das Moorhuhn abzuschießen.
 * Wenn man diese einhält, bekommt man 10 Punkte. Wenn man es sogar in weniger als 2/3 dieser Zeit schafft, bekommt man 30 Punkte.
 * Wenn man es sogar in weniger als 1/3 dieser Zeit schafft, bekommt man 50 Punkte.
 * Die Variable wird global eingestellt, daher braucht die Funktion den Wert nicht zurückgeben.
 */
void rt_settings() {
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Max. Reaktionszeit");
  lcd.setCursor(0,1);
  lcd.print("(fuer 10 Punkte)");
  lcd.setCursor(9,2);
  lcd.print(react_time,1);
  lcd.print("s");
  lcd.setCursor(0,3);
  lcd.print("--     Zurueck    ++");
  while(read_buttons()!=t2) {
    while(read_buttons()==no_t) {}
    lcd_key=read_buttons();
    if (lcd_key==t1 && react_time>1.0) {
      react_time--;
    } else if( lcd_key==t3 && react_time<11.0) {
      react_time++;
    }
    lcd.setCursor(9,2);
    lcd.print("     ");
    lcd.setCursor(9,2);
    lcd.print(react_time,1);
    lcd.print("s");
    delay(500);
  }
}

/* Die Bewegungshäufigkeit gibt an, wie oft sich das Huhn bewegt. Dabei bewegt es sich weniger oft, wenn die Bewegungshäufigkeit hbew größer wird.
 *  hbew gibt an, dass sich das Moorhuhn ein Mal alle hbew*100 Millisekunden bewegt. Wenn hbew = 10 ist, bewegt sich das Moorhuhn also einmal in 1s.
 *  Wohin sich das Moorhuhn bewegt, wird im Spiel zufällig ausgewählt. 
 */
void bh_settings() {
  int bh=hbew*100;
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Bewegungshaeufigkeit");
  lcd.setCursor(0,1);
  lcd.print("Huhn bewegt sich");
  lcd.setCursor(0,2);
  lcd.print("ein Mal in");
  lcd.setCursor(11,2);
  lcd.print(bh);
  lcd.print("ms");
  lcd.setCursor(0,3);
  lcd.print("--     Zurueck    ++");
  delay(500);
  while(read_buttons()!=t2) {
    while(read_buttons()==no_t) {}
    lcd_key=read_buttons();
    if (lcd_key==t1 && hbew>1) {
      hbew--;
    } else if( lcd_key==t3 && hbew<20) {
      hbew++;
    }
    bh=hbew*100;
    lcd.setCursor(11,2);
    lcd.print("      ");
    lcd.setCursor(11,2);
    lcd.print(bh);
    lcd.print("ms");
    delay(500);
  }
}

void loop() {
  // Startbildschirm aufbauen
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print(" Moorhuhn-Lasertag! ");
  lcd.setCursor(0,1);
  lcd.print("1: Moorhuhn Classic");
  lcd.setCursor(0,2);
  lcd.print("2: Kill 'em all");
  lcd.setCursor(0,3);
  lcd.print("3: Einstellungen");
  delay(500);
  while(read_buttons()==no_t) {}
  lcd_key=read_buttons();
  switch (lcd_key) {
    case t1:
    { // Es wurde das Spiel Classic gewählt. Bevor es losgeht, muss die Anzahl der Spieler festgelegt werden.
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("Klassisches Moorhuhn");
      lcd.setCursor(0,1);
      lcd.print("   Spieleranzahl:   ");
      while(read_buttons()!=t2) {
        lcd.setCursor(9,2);
        lcd.print(sp);
        if (sp==1) {
          lcd.setCursor(0,3);
          lcd.print("                    ");
          lcd.setCursor(9,3);
          lcd.print("Ok      ++");
        } else if (sp==2) {
          lcd.setCursor(0,3);
          lcd.print("                    ");
          lcd.setCursor(0,3);
          lcd.print("--       Ok");
        }
        while (read_buttons()==no_t) {}
        lcd_key=read_buttons();
        if (lcd_key==t1 && sp>1) {
          sp--;
        } else if (lcd_key==t3 && sp<2) {
          sp++;
        } else if (lcd_key==t2) {
          break;
        }
      }
      // Es wurde die Spieleranzahl festgelegt und es kann losgehen
      classic(sp, react_time, hbew); // eigentlich ist nicht nötig, die Variablen mitzugeben, weil sie global definiert sind?! 
                                     // Aber es ist leichter verständlich, sie mitzugeben
      // Das Spiel ist zu Ende und es werden die Ergebnisse ausgegeben.
      lcd.clear();
      lcd.setCursor(0,0);
      if (sp==1) {
        lcd.print("Ergebnis:");
        lcd.setCursor(0,1);
        lcd.print("Du hast ");
        //lcd.print(counter[1]);
        lcd.print(a);
        lcd.print(" Punkte!");
        lcd.setCursor(0,2);
        lcd.print("       Super!      ");
      } else {
        lcd.print("Ergebnisse:");
        lcd.setCursor(0,1);
        lcd.print("Spieler1: ");
        //lcd.print(counter[1]);
        lcd.print(a);
        lcd.print("Pkt");
        lcd.setCursor(0,2);
        lcd.print("Spieler2: ");
        //lcd.print(counter[2]);
        lcd.print(b);
        lcd.print("Pkt");
      }
      for (int i=0; i<3; i++) { // LED-Kette blinkt 3 Mal
          digitalWrite(led_kette, LOW);
          delay(500);
          digitalWrite(led_kette, HIGH);
          delay(500);
      }
      lcd.setCursor(0,3);
      lcd.print("Weiter in...");
      for (int i=5; i>0; i--) {
        lcd.setCursor(13,3);
        lcd.print(i);
        delay(1000);
      }
      break;
    }
    case t2:
    {
      // Es wurde das Spiel Kill 'em all ausgewählt
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("Error 404: Game not");
      lcd.setCursor(0,1);
      lcd.print("found.");
      lcd.setCursor(0,3);
      lcd.print("Weiter in...");
      for (int i=5; i>0; i--) {
        lcd.setCursor(13,3);
        lcd.print(i);
        delay(1000);
      }
      break;
    }
    case t3:
    { // Es wurde der Einstellungsdialog aufgerufen
      lcd.clear();
      lcd.setCursor(0,0);
      lcd.print("    Einstellungen   ");
      lcd.setCursor(0,1);
      lcd.print("1: Reaktionszeit");
      lcd.setCursor(0,2);
      lcd.print("2: Bewegung");
      lcd.setCursor(6,3);
      lcd.print("Zurueck");
      delay(500);
      while (read_buttons()==no_t) {}
      lcd_key=read_buttons();
      if (lcd_key==t1) {
        rt_settings();
      } else if (lcd_key==t2) {
        bh_settings();
      } else if(lcd_key==t3) {
        break;
      }
    }
  }
}









